﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataPreprocessing {
  [Item("Data Grid", "Represents a data grid.")]
  [StorableClass]
  public class DataGridContent : PreprocessingContent, IStringConvertibleMatrix, IViewShortcut {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Table; }
    }

    [Storable]
    public ManipulationLogic ManipulationLogic { get; private set; }

    public int Rows {
      get { return PreprocessingData.Rows; }
      set { }
    }

    public int Columns {
      get { return PreprocessingData.Columns; }
      set { }
    }

    public IEnumerable<string> ColumnNames {
      get { return PreprocessingData.VariableNames; }
      set { }
    }

    public IEnumerable<string> RowNames {
      get { return Enumerable.Range(1, Rows).Select(n => n.ToString()); }
      set { throw new NotSupportedException(); }
    }

    public bool SortableView {
      get { return true; }
      set { throw new NotSupportedException(); }
    }

    public bool ReadOnly {
      get { return false; }
    }

    public IDictionary<int, IList<int>> Selection {
      get { return PreprocessingData.Selection; }
      set { PreprocessingData.Selection = value; }
    }

    #region Constructor, Cloning & Persistence
    public DataGridContent(IFilteredPreprocessingData preprocessingData, ManipulationLogic manipulationLogic)
      : base(preprocessingData) {
      ManipulationLogic = manipulationLogic;
    }

    public DataGridContent(DataGridContent original, Cloner cloner)
      : base(original, cloner) {
      ManipulationLogic = cloner.Clone(original.ManipulationLogic);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new DataGridContent(this, cloner);
    }

    [StorableConstructor]
    protected DataGridContent(bool deserializing)
      : base(deserializing) { }
    #endregion

    public void DeleteRows(IEnumerable<int> rows) {
      PreprocessingData.DeleteRowsWithIndices(rows);
    }

    public void DeleteColumn(int column) {
      PreprocessingData.DeleteColumn(column);
    }

    public bool Validate(string value, out string errorMessage, int columnIndex) {
      return PreprocessingData.Validate(value, out errorMessage, columnIndex);
    }

    public string GetValue(int rowIndex, int columnIndex) {
      return PreprocessingData.GetCellAsString(columnIndex, rowIndex);
    }

    public bool SetValue(string value, int rowIndex, int columnIndex) {
      return PreprocessingData.SetValue(value, columnIndex, rowIndex);
    }

    public event DataPreprocessingChangedEventHandler Changed {
      add { PreprocessingData.Changed += value; }
      remove { PreprocessingData.Changed -= value; }
    }

    #region unused stuff/not implemented but necessary due to IStringConvertibleMatrix
#pragma warning disable 0067
    // Is not used since DataGridContentView overrides dataGridView_CellValidating and uses 
    // DataGridLogic#Validate(string value, out string errorMessage, int columnIndex)
    public bool Validate(string value, out string errorMessage) {
      errorMessage = string.Empty;
      return true;
    }

    public event EventHandler ColumnsChanged;
    public event EventHandler RowsChanged;
    public event EventHandler ColumnNamesChanged;
    public event EventHandler RowNamesChanged;
    public event EventHandler SortableViewChanged;
    public event EventHandler<EventArgs<int, int>> ItemChanged;
    public event EventHandler Reset;
#pragma warning restore 0067
    #endregion
  }
}
