﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Drawing;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.DataPreprocessing {
  [Item("Manipulation", "Represents the available manipulations on a data set.")]
  [StorableClass]
  public class ManipulationContent : PreprocessingContent, IViewShortcut {
    public static new Image StaticItemImage {
      get { return HeuristicLab.Common.Resources.VSImageLibrary.Method; }
    }

    [Storable]
    public ManipulationLogic ManipulationLogic { get; private set; }

    #region Constructor, Cloning & Persistence
    public ManipulationContent(IFilteredPreprocessingData preprocessingData, ManipulationLogic manipulationLogic)
      : base(preprocessingData) {
      ManipulationLogic = manipulationLogic;
    }

    public ManipulationContent(ManipulationContent original, Cloner cloner) :
      base(original, cloner) {
      ManipulationLogic = cloner.Clone(original.ManipulationLogic);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new ManipulationContent(this, cloner);
    }

    [StorableConstructor]
    protected ManipulationContent(bool deserializing)
      : base(deserializing) { }
    #endregion
  }
}
